@@ -1,9 +1,13 @@
-#!/usr/bin/env python2.4
+#!/usr/bin/env python
+"""
+Copyright (C) 2012 Roman Mohr <roman@fenkhuber.at>
+"""
+
 """static - A stupidly simple WSGI way to serve static (or mixed) content.
 
 (See the docstrings of the various functions and classes.)
 
-Copyright (C) 2006-2008 Luke Arno - http://lukearno.com/
+Copyright (C) 2006-2009 Luke Arno - http://lukearno.com/
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -18,8 +22,8 @@ Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to:
 
-The Free Software Foundation, Inc., 
-51 Franklin Street, Fifth Floor, 
+The Free Software Foundation, Inc.,
+51 Franklin Street, Fifth Floor,
 Boston, MA  02110-1301, USA.
 
 Luke Arno can be found at http://lukearno.com/
@@ -27,48 +31,84 @@ Luke Arno can be found at http://lukearno.com/
 """
 
 import mimetypes
-import rfc822
+import email.utils as rfc822
 import time
 import string
 import sys
 from os import path, stat
 from wsgiref import util
+from wsgiref.headers import Headers
 from wsgiref.simple_server import make_server
 from optparse import OptionParser
 
-try: from pkg_resources import resource_filename, Requirement
-except: pass
+try:
+    from pkg_resources import resource_filename, Requirement
+except:
+    pass
 
-try: import kid
-except: pass
+try:
+    import kid
+except:
+    pass
+
+try:
+    from genshi.template import MarkupTemplate
+except:
+    pass
 
 
-class MagicError(Exception): pass
+class MagicError(Exception):
+    pass
 
 
+def _encode(string, encoding):
+    if sys.version_info[0] > 2:
+        return string.encode(encoding=encoding, errors='strict')
+    else:
+        if type(u'') == type(string):
+            string = string.encode(encoding)
+        return string
+
+def _decode(string, encoding):
+    if sys.version_info[0] > 2:
+        return string.decode(encoding=encoding, errors='strict')
+    else:
+        return string
+
+def _open(filename, encoding):
+    if sys.version_info[0] > 2:
+        return open(filename, 'r', encoding=encoding, errors='strict') 
+    else:
+        return open(filename, 'rb')
+
 class StatusApp:
-    """A WSGI app that just returns the given status."""
-    
-    def __init__(self, status, message=None):
+
+    """Used by WSGI apps to return some HTTP status."""
+
+    def __init__(self, status, message=None, encoding=sys.getdefaultencoding()):
         self.status = status
+        self.encoding = encoding
         if message is None:
             self.message = status
         else:
             self.message = message
-        
+
     def __call__(self, environ, start_response, headers=[]):
+        if self.message:
+            Headers(headers).add_header('Content-type', 'text/plain')
         start_response(self.status, headers)
-        if environ['REQUEST_METHOD'] == 'GET':
-            return [self.message]
+        if environ['REQUEST_METHOD'] == 'HEAD':
+            return [_encode("",self.econding)]
         else:
-            return [""]
+            return [_encode(self.message, self.encoding)]
 
 
 class Cling(object):
+
     """A stupidly simple way to serve static content via WSGI.
-    
+
     Serve the file of the same path as PATH_INFO in self.datadir.
-    
+
     Look up the Content-type in self.content_types by extension
     or use 'text/plain' if the extension is not found.
 
@@ -85,19 +125,22 @@ class Cling(object):
     def __init__(self, root, **kw):
         """Just set the root and any other attribs passes via **kw."""
         self.root = root
-        for k, v in kw.iteritems():
+        self.encoding = sys.getdefaultencoding()
+
+        for k, v in kw.items():
             setattr(self, k, v)
 
     def __call__(self, environ, start_response):
         """Respond to a request when called in the usual WSGI way."""
         if environ['REQUEST_METHOD'] not in ('GET', 'HEAD'):
-            return self.method_not_allowed(environ, start_response)
+            headers = [('Allow', 'GET, HEAD')]
+            return self.method_not_allowed(environ, start_response, headers)
         path_info = environ.get('PATH_INFO', '')
         full_path = self._full_path(path_info)
         if not self._is_under_root(full_path):
             return self.not_found(environ, start_response)
         if path.isdir(full_path):
-            if full_path[-1] <> '/' or full_path == self.root:
+            if full_path[-1] != '/' or full_path == self.root:
                 location = util.request_uri(environ, include_query=False) + '/'
                 if environ.get('QUERY_STRING'):
                     location += '?' + environ.get('QUERY_STRING')
@@ -123,10 +166,11 @@ class Cling(object):
             start_response("200 OK", headers)
             if environ['REQUEST_METHOD'] == 'GET':
                 return self._body(full_path, environ, file_like)
+                      
             else:
                 return ['']
-        except (IOError, OSError), e:
-            print e
+        except (IOError, OSError) as e:
+            print(e)
             return self.not_found(environ, start_response)
 
     def _full_path(self, path_info):
@@ -136,7 +180,7 @@ class Cling(object):
     def _is_under_root(self, full_path):
         """Guard against arbitrary file retrieval."""
         if (path.abspath(full_path) + path.sep)\
-            .startswith(path.abspath(self.root) + path.sep):
+                .startswith(path.abspath(self.root) + path.sep):
             return True
         else:
             return False
@@ -165,30 +209,33 @@ def iter_and_close(file_like, block_size):
     while 1:
         try:
             block = file_like.read(block_size)
-            if block: yield block
-            else: raise StopIteration
-        except StopIteration, si:
+            if block:
+                yield block
+            else:
+                raise StopIteration
+        except StopIteration as si:
             file_like.close()
-            return 
+            return
 
 
 def cling_wrap(package_name, dir_name, **kw):
     """Return a Cling that serves from the given package and dir_name.
-    
+
     This uses pkg_resources.resource_filename which is not the
-    recommended way, since it extracts the files. 
-    
-    I think this works fine unless you have some _very_ serious 
-    requirements for static content, in which case you probably 
+    recommended way, since it extracts the files.
+
+    I think this works fine unless you have some _very_ serious
+    requirements for static content, in which case you probably
     shouldn't be serving it through a WSGI app, IMHO. YMMV.
     """
     resource = Requirement.parse(package_name)
     return Cling(resource_filename(resource, dir_name), **kw)
- 
+
 
 class Shock(Cling):
+
     """A stupidly simple way to serve up mixed content.
-    
+
     Serves static content just like Cling (it's superclass)
     except that it process content with the first matching
     magic from self.magics if any apply.
@@ -198,17 +245,17 @@ class Shock(Cling):
     If you are using Shock with the StringMagic class for instance:
 
     shock = Shock('/data', magics=[StringMagic(food='cheese')])
-    
+
     Let's say you have a file called /data/foo.txt.stp containing one line:
 
     "I love to eat $food!"
-    
+
     When you do a GET on /foo.txt you will see this in your browser:
 
     "I love to eat cheese!"
 
     This is really nice if you have a color variable in your css files or
-    something trivial like that. It seems silly to create or change a 
+    something trivial like that. It seems silly to create or change a
     handful of objects for a couple of dynamic bits of text.
     """
 
@@ -226,7 +273,7 @@ class Shock(Cling):
         if path.exists(full_path):
             return full_path
         else:
-            for magic in self.magics: 
+            for magic in self.magics:
                 if path.exists(magic.new_path(full_path)):
                     return magic.new_path(full_path)
             else:
@@ -236,7 +283,7 @@ class Shock(Cling):
         """Guess the mime type magically or using the mimetypes module."""
         magic = self._match_magic(full_path)
         if magic is not None:
-            return (mimetypes.guess_type(magic.old_path(full_path))[0] 
+            return (mimetypes.guess_type(magic.old_path(full_path))[0]
                     or 'text/plain')
         else:
             return mimetypes.guess_type(full_path)[0] or 'text/plain'
@@ -254,7 +301,7 @@ class Shock(Cling):
         """Return the appropriate file object."""
         magic = self._match_magic(full_path)
         if magic is not None:
-            return magic.file_like(full_path)
+            return magic.file_like(full_path, self.encoding)
         else:
             return open(full_path, 'rb')
 
@@ -262,17 +309,19 @@ class Shock(Cling):
         """Return an iterator over the body of the response."""
         magic = self._match_magic(full_path)
         if magic is not None:
-            return magic.body(environ, file_like)
+            return [_encode(s,self.encoding) for s in magic.body(environ,
+                file_like)]
         else:
             way_to_send = environ.get('wsgi.file_wrapper', iter_and_close)
             return way_to_send(file_like, self.block_size)
 
 
 class BaseMagic(object):
+
     """Base class for magic file handling.
 
     Really a do nothing if you were to use this directly.
-    
+
     In a strait forward case you would just override .extension and body().
     (See StringMagic in this module for a simple example of subclassing.)
 
@@ -295,7 +344,7 @@ class BaseMagic(object):
         if self.matches(full_path):
             return full_path[:-len(self.extension)]
         else:
-            raise MagicError, "Path does not match this magic."
+            raise MagicError("Path does not match this magic.")
 
     def matches(self, full_path):
         """Check that path ends with self.extension."""
@@ -307,9 +356,9 @@ class BaseMagic(object):
         mtime = int(time.time())
         return str(mtime), rfc822.formatdate(mtime)
 
-    def file_like(self, full_path):
+    def file_like(self, full_path, encoding):
         """Return a file object for path."""
-        return open(full_path, 'rb')
+        return _open(full_path, encoding)
 
     def body(self, environ, file_like):
         """Return an iterator over the body of the response."""
@@ -317,8 +366,9 @@ class BaseMagic(object):
 
 
 class StringMagic(BaseMagic):
+
     """Magic to replace variables in file contents using string.Template.
-    
+
     Using this requires Python2.4.
     """
 
@@ -331,7 +381,7 @@ class StringMagic(BaseMagic):
 
     def body(self, environ, file_like):
         """Pass environ and self.variables in to template.
-        
+
         self.variables overrides environ so that suprises in environ don't
         cause unexpected output if you are passing a value in explicitly.
         """
@@ -345,23 +395,43 @@ class StringMagic(BaseMagic):
 
 
 class KidMagic(StringMagic):
+
     """Like StringMagic only using the Kid templating language.
-    
+
     Using this requires Kid: http://kid.lesscode.org/
     """
-    
+
     extension = '.kid'
-    
+
     def body(self, environ, full_path):
         """Pass environ and **self.variables into the template."""
-        template = kid.Template(file=full_path, 
-                                environ=environ, 
+        template = kid.Template(file=full_path,
+                                environ=environ,
                                 **self.variables)
         return [template.serialize()]
 
 
+class GenshiMagic(StringMagic):
+
+    """Like StringMagic only using the Genshi templating language.
+
+    Using this requires Genshi
+    """
+
+    extension = '.genshi'
+
+    def body(self, environ, full_path):
+        """Pass environ and **self.variables into the template."""
+
+        template = MarkupTemplate(full_path.read())
+        variables = self.variables.copy()
+        variables["environ"] = environ
+        return [template.generate(**variables)
+                .render('html', doctype='html')]
+
+
 def command():
-    parser = OptionParser(usage="%prog DIR [HOST][:][PORT]", 
+    parser = OptionParser(usage="%prog DIR [HOST][:][PORT]",
                           version="static 0.3.6")
     options, args = parser.parse_args()
     if len(args) in (1, 2):
@@ -387,8 +457,8 @@ def command():
         app = Cling(args[0])
         try:
             make_server(host, port, app).serve_forever()
-        except KeyboardInterrupt, ki:
-            print "Cio, baby!"
+        except KeyboardInterrupt as ki:
+            print("Cio, baby!")
         except:
             sys.exit("Problem initializing server.")
     else:
@@ -397,14 +467,15 @@ def command():
 
 
 def test():
-    magics = StringMagic(title="String Test"), KidMagic(title="Kid Test")
+    from wsgiref.validate import validator
+    magics = (StringMagic(title="String Test"),
+              KidMagic(title="Kid Test"), GenshiMagic(title="Genshi Test"))
     app = Shock('testdata/pub', magics=magics)
     try:
-        make_server('localhost', 9999, app).serve_forever()
-    except KeyboardInterrupt, ki:
-        print "Cio, baby!"
+        make_server('localhost', 9999, validator(app)).serve_forever()
+    except KeyboardInterrupt as ki:
+        print("Ciao, baby!")
 
 
 if __name__ == '__main__':
     test()
-
